import math

from django import template
from django.contrib.contenttypes.models import ContentType

from rating.models import RatingSettings, UserRating, Rating

register = template.Library()


def get_object_info(obj):
    object_info = {
        'app_name': type(obj)._meta.app_label,
        'model_name': type(obj).__name__,
        'content_type': ContentType.objects.get_for_model(obj),
        'object_id': obj.id
    }
    return object_info


@register.inclusion_tag('rating/rating/rating.html')
def render_rating(request, obj, settings_slug):
    context = {
        'object': obj,
        'request': request,
        'settings': RatingSettings.objects.get(slug=settings_slug),
        'object_info': get_object_info(obj)
    }
    return context


@register.simple_tag
def get_rating(object_info, settings):
    rating = Rating.objects.get_or_create(object_id=object_info['object_id'], content_type=object_info['content_type'], settings=settings)[0]
    return rating


@register.simple_tag
def get_user_rate(request, rating):
    if request.user.is_authenticated and UserRating.objects.filter(user=request.user, rating=rating).exists():
        user_rating = UserRating.objects.get(user=request.user, rating=rating)
        rate = math.ceil(user_rating.rate * rating.settings.rates)
        return rate
    else:
        return None


@register.inclusion_tag('rating/info/info_extender.html')
def render_rating_info(request, obj, settings_slug, custom_template='rating/rating/rating_info.html'):
    settings = RatingSettings.objects.get(slug=settings_slug)
    object_info = get_object_info(obj)
    rating = get_rating(object_info, settings)
    user_rating = UserRating.objects.get(user=request.user, rating=rating) if request.user.is_authenticated and UserRating.objects.filter(user=request.user, rating=rating).exists() else None
    context = {
        'request': request,
        'template': custom_template,
        'settings': settings,
        'urlhash': rating.urlhash,
        'count': rating.count,
        'average': round(rating.average * settings.rates, 2),
        'rate': math.ceil(user_rating.rate * settings.rates) if user_rating is not None else None
    }
    return context


@register.inclusion_tag('rating/utils/IMPORTS.html')
def render_rating_import():
    pass


@register.inclusion_tag('rating/utils/SCRIPTS.html')
def render_rating_script():
    pass



import io
import os
from pathlib import Path

import requests
import time
from django.core.files import File
from django.core.files.uploadedfile import InMemoryUploadedFile
from django.db import migrations


def save_image_from_url(url):
    response = requests.get(url)
    file = io.BytesIO(response.content)
    filename = os.path.basename(url)
    name, ext = os.path.splitext(filename)
    file_name = f'{name}-{time.time()}{ext}'
    uploaded_file = InMemoryUploadedFile(file, None, file_name, 'image/svg', file.getbuffer().nbytes, None)
    return uploaded_file, file_name


def InitialRatingSettings(apps, schema_editor):
    RatingSettings = apps.get_model('rating', 'RatingSettings')

    print('Fetching images from web')

    # STAR RATING CONFIG
    file, file_name = save_image_from_url(url="https://raw.githubusercontent.com/mahyar-amiri/django-rating-system/master/rating/static/rating/img/star.svg")
    RatingSettings.objects.create(name='Default Config', slug='default-config', icon=File(file, file_name))

    # LIKE RATING CONFIG
    file_path = Path('rating/static/img/heart.svg')
    file, file_name = save_image_from_url(url="https://raw.githubusercontent.com/mahyar-amiri/django-rating-system/master/rating/static/rating/img/heart.svg")
    RatingSettings.objects.create(name='Like Config', slug='like-config', from_zero=False, rates=1, icon=File(file, file_name))


class Migration(migrations.Migration):
    dependencies = [
        ('rating', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(InitialRatingSettings),
    ]



from random import choice
from string import ascii_lowercase

from django.contrib.auth import get_user_model
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.html import format_html

from rating import settings

User = get_user_model()


class RatingSettings(models.Model):
    name = models.CharField(max_length=30)
    slug = models.SlugField(primary_key=True, help_text='This value will be used in render_rating tag')
    from_zero = models.BooleanField(default=False, help_text='Rating starts from 0 or 1')
    RATES_CHOICES = ((1, 1), (3, 3), (5, 5), (10, 10))
    rates = models.SmallIntegerField(default=5, choices=RATES_CHOICES)
    icon = models.FileField(upload_to='rating_settings')
    height = models.CharField(max_length=10, default='2rem')

    class Meta:
        verbose_name = 'Rating Settings'
        verbose_name_plural = 'Rating Settings'

    def __str__(self):
        return f'{self.name}'

    def source_file(self):
        return format_html(f"<img style='height:20px' src='{self.icon.url}'>") if self.icon else 'X'


class Rating(models.Model):
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey('content_type', 'object_id')
    urlhash = models.CharField(max_length=50, unique=True, editable=False)
    average = models.FloatField(default=0)
    count = models.PositiveBigIntegerField(default=0)
    settings = models.ForeignKey(RatingSettings, null=True, on_delete=models.SET_NULL)

    def __str__(self):
        return f'{self.content_type} - {self.content_object} - {self.urlhash}'

    @staticmethod
    def generate_urlhash():
        return ''.join(choice(ascii_lowercase) for _ in range(settings.URLHASH_LENGTH))

    def set_unique_urlhash(self):
        if not self.urlhash:
            self.urlhash = self.generate_urlhash()
            while self.__class__.objects.filter(urlhash=self.urlhash).exists():
                self.urlhash = self.generate_urlhash()

    def save(self, *args, **kwargs):
        self.set_unique_urlhash()
        super(Rating, self).save(*args, **kwargs)


class UserRating(models.Model):
    user = models.ForeignKey(User, on_delete=models.CASCADE, related_name='ratings')
    rate = models.FloatField()  # ON SCALE OF 0 TO 1
    rating = models.ForeignKey(Rating, on_delete=models.CASCADE, related_name='ratings')
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

    class Meta:
        ordering = ('-created',)

    def __str__(self):
        return f'[{self.user}] {self.rate}'



from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Rating',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
                ('urlhash', models.CharField(editable=False, max_length=50, unique=True)),
                ('average', models.FloatField(default=0)),
                ('count', models.PositiveBigIntegerField(default=0)),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype')),
            ],
        ),
        migrations.CreateModel(
            name='RatingSettings',
            fields=[
                ('name', models.CharField(max_length=30)),
                ('slug', models.SlugField(help_text='This value will be used in render_rating tag', primary_key=True, serialize=False)),
                ('from_zero', models.BooleanField(default=False, help_text='Rating starts from 0 or 1')),
                ('rates', models.SmallIntegerField(choices=[(1, 1), (3, 3), (5, 5), (10, 10)], default=5)),
                ('icon', models.FileField(upload_to='rating_settings')),
                ('height', models.CharField(default='2rem', max_length=10)),
            ],
            options={
                'verbose_name': 'Rating Settings',
                'verbose_name_plural': 'Rating Settings',
            },
        ),
        migrations.CreateModel(
            name='UserRating',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('rate', models.FloatField()),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('rating', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='ratings', to='rating.rating')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='ratings', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ('-created',),
            },
        ),
        migrations.AddField(
            model_name='rating',
            name='settings',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='rating.ratingsettings'),
        ),
    ]



from django.urls import path

from rating import views

app_name = 'rating'
urlpatterns = [
    path('create/', views.UserRatingCreate.as_view(), name='create'),
    path('info/<str:urlhash>/', views.RatingInfo.as_view(), name='info'),
]



import json
import math
import statistics

from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.contenttypes.models import ContentType
from django.http import HttpResponse, HttpResponseBadRequest
from django.shortcuts import render
from django.views import View

from rating.models import Rating, RatingSettings, UserRating


class UserRatingCreate(View, LoginRequiredMixin):
    def post(self, request, *args, **kwargs):
        try:
            data = json.loads(request.body)
            app_name = data.get('app_name', None)
            model_name = data.get('model_name', None)
            object_id = data.get('object_id', None)
            content_type = ContentType.objects.get(app_label=app_name, model=model_name.lower())
            settings_slug = data.get('settings_slug', None)
            rating_settings = RatingSettings.objects.get(slug=settings_slug)

            rating = Rating.objects.get_or_create(object_id=object_id, content_type=content_type, settings=rating_settings)[0]
            rate = int(data.get('rate', None))
            if UserRating.objects.filter(user=request.user, rating=rating).exists():
                user_rating = UserRating.objects.get(user=request.user, rating=rating)
                if rate == -1:
                    user_rating.delete()
                else:
                    user_rating.rate = rate / rating_settings.rates
                    user_rating.save()
            else:
                UserRating.objects.create(user=request.user, rating=rating, rate=rate / rating_settings.rates)

            # RATING UPDATES
            rating.count = rating.ratings.count()
            # math.ceil(user_rating.rate * settings.rates)
            rates = [r.rate for r in rating.ratings.all()]
            rating.average = round(statistics.mean(rates), 2) if rates else 0
            rating.save()

            return HttpResponse({rating.urlhash}, status=200)
        except Exception as e:
            print(e)
            return HttpResponseBadRequest()


class RatingInfo(View, LoginRequiredMixin):
    def get(self, request, urlhash, *args, **kwargs):
        rating = Rating.objects.get(urlhash=urlhash)
        user_rating = UserRating.objects.get(user=request.user, rating=rating) if UserRating.objects.filter(user=request.user, rating=rating).exists() else None
        custom_template = request.GET.get('custom_template', None)
        context = {
            'request': request,
            'settings': rating.settings,
            'template': custom_template,
            'count': rating.count,
            'average': round(rating.average * rating.settings.rates, 2),
            'rate': math.ceil(user_rating.rate * rating.settings.rates) if user_rating is not None else None
        }
        return render(request, 'rating/info/info_extender.html', context=context)



import math
from django.contrib import admin

from rating.models import Rating, UserRating, RatingSettings


class RatingAdmin(admin.ModelAdmin):
    list_display = ('content_obj', 'urlhash', 'count', 'average', 'total_average', 'settings')
    readonly_fields = ('content_type', 'urlhash', 'object_id', 'content_object', 'average', 'count', 'settings')

    def content_obj(self, obj):
        return f'{obj.content_type} - {obj.content_object}'

    content_obj.short_description = 'Content Object'

    def total_average(self, obj):
        return f'{round(obj.average * obj.settings.rates, 2)}/{obj.settings.rates}'

    total_average.short_description = 'Total Average'


class UserRatingAdmin(admin.ModelAdmin):
    list_display = ('user', 'rate', 'user_rate', 'rating')
    readonly_fields = ('user', 'rate', 'rating')

    def user_rate(self, obj):
        return f'{math.ceil(obj.rate * obj.rating.settings.rates)}/{obj.rating.settings.rates}'

    user_rate.short_description = 'Total Rate'


class SettingsAdmin(admin.ModelAdmin):
    list_display = ('name', 'slug', 'source_file')
    search_fields = ('name', 'slug')
    prepopulated_fields = {'slug': ('name',)}


admin.site.register(Rating, RatingAdmin)
admin.site.register(UserRating, UserRatingAdmin)
admin.site.register(RatingSettings, SettingsAdmin)
